import { IsMapOfStrings } from './decorators/is-map-of-strings.decorator';
import { DLCommonsModule } from './dl-commons.module';
import { HandledException } from './exceptions/custom.exception';
import { validationExceptionFactory } from './factories/exception.factory';
import { BaseExceptionsFilter } from './filters/base-exception.filter';
import { AccessTokenGuard } from './guards/access-token.guard';
import { AllowPermissions, PermissionGuard } from './guards/permission.guard';
import {
    APP__PORT,
    APP__DEBUG,
    APP__ACCESS_TOKEN,
    MONGO__CS,
    REDIS__CS,
    KAFKA__BROKERS,
    APP__MONGO__CS,
    DL_SESSIONS__MONGO__CS,
} from './models/consts/env-vars.const';
import { I18nDTO } from './models/dtos/commons/i18n.dto';
import {
    IMongoPage,
    RESTError,
    RESTPage,
    RESTPaging,
    RESTQueryParams,
    RESTRequestBody,
    RESTResponseBody,
    RESTResult,
    RESTSort,
    SortDirection,
} from './models/dtos/wrappers/rest-wrapper.class';
import { ContactType } from './models/enums/contact-type.enum';
import { RecordState } from './models/enums/record-state.enum';
import { UserVerificationLevel } from './models/enums/user-verifiation-level.enum';
import { IBase } from './models/interfaces/base.interface';
import { IContact } from './models/interfaces/contact.interface';
import { IDictionary } from './models/interfaces/dictionary.interface';
import { II18n } from './models/interfaces/i18n.interface';
import { IIdentifable } from './models/interfaces/identifable.interface';
import { IPermission } from './models/interfaces/permission.interface';
import { IPerson } from './models/interfaces/person.interface';
import { IRecord } from './models/interfaces/record.interface';
import { IResidence } from './models/interfaces/residence.interface';
import { IRole } from './models/interfaces/role.interface';
import { ISession } from './models/interfaces/session.interface';
import { ISessionUser } from './models/interfaces/session-user.interface';
import { Base, BaseDocument, BaseSchema } from './models/schemas/base.schema';
import { Contact, ContactDocument, ContactSchema } from './models/schemas/embedded/contact.schema';
import { DictionaryEmbedded, DictionaryEmbeddedDocument, DictionaryEmbeddedSchema } from './models/schemas/embedded/dictionary-embedded.schema';
import { I18n, I18nSchema, I18nDocument } from './models/schemas/embedded/i18n.embedded';
import { Person, PersonSchema, PersonDocument } from './models/schemas/embedded/person.schema';
import { Record, RecordSchema, RecordDocument } from './models/schemas/embedded/record.embedded';
import { Residence, ResidenceDocument, ResidenceSchema } from './models/schemas/embedded/residence.schema';
import { Identifable, IdentifableDocument, IdentifableSchema } from './models/schemas/identifable.schema';
import { MongoIdPipe } from './pipes/mongo-id.pipe';
import { MongoIdsPipe } from './pipes/mongo-ids.pipe';
import { SessionsService } from './services/sessions.service';
import { transformers } from './transformers/common.transformer';
import { IUser } from './models/interfaces/user.interface';
import { SessionUser, SessionUserDocument, SessionUserSchema } from './models/schemas/embedded/session-user.schema';
import { Session, SessionDocument, SessionSchema } from './models/schemas/session.schema';
import { DictionariesService } from './services/dictionaries.service';
import { IParent } from './models/interfaces/parent.interface';
import { IGroupEmbedded } from './models/interfaces/group-embedded.interface';
import { IStudentEmbedded } from './models/interfaces/student-embedded.interface';
import { StudentEmbedded, StudentEmbeddedDocument, StudentEmbeddedSchema } from './models/schemas/embedded/student-embedded.schema';
import { ITeacher } from './models/interfaces/teacher.interface';
import { WeekDay } from './models/enums/week-day.enum';
import { IGroupSession, ITime } from './models/interfaces/group-session.interface';
import { GroupState } from './models/enums/group-state.enum';
import { GroupSessionSchema, GroupSessionDocument, GroupSession } from './models/schemas/embedded/group-session.schema';
import { TeacherSchema, TeacherDocument, Teacher } from './models/schemas/embedded/teacher.schema';
import { ISMTPAPIConfig } from './models/interfaces/smtp-api-config.interface';
import { SMTPService } from './services/smtp.service';
import { Time, TimeDocument, TimeSchema } from './models/schemas/embedded/time.schema';
import { FileLogInitType, S3FileState } from './models/enums/s3.enum';
import { IS3, ILogInitiator, IS3File, IFileState, IFileRecord, IFileMeta, IFileLog, IFileEmbedded } from './models/interfaces/s3.interface';
import {
    S3,
    S3Schema,
    S3Document,
    LogInitiatorDocument,
    LogInitiatorSchema,
    LogInitiator,
    FileLog,
    FileLogSchema,
    FileLogDocument,
    FileMetaDocument,
    FileMetaSchema,
    FileMeta,
    FileState,
    FileStateSchema,
    FileStateDocument,
    FileRecord,
    FileRecordSchema,
    FileRecordDocument,
    S3File,
    S3FileSchema,
    S3FileDocument,
    FileEmbedded,
    FileEmbeddedDocument,
    FileEmbeddedSchema,
} from './models/schemas/embedded/s3.schema';
import { IUserMeta } from './models/interfaces/user-meta.interface';
import { ICropArea } from './models/interfaces/crop-area.interface';
import { CropArea, CropAreaDocument, CropAreaSchema } from './models/schemas/embedded/crop-area.schema';
import { AttachementType } from './models/enums/attachement-type.enum';
import { IAttachement } from './models/interfaces/attachement.interface';
import { Attachement, AttachementDocument, AttachementSchema } from './models/schemas/embedded/attachement.schema';
import { Picture, PictureSchema, PictureDocument } from './models/schemas/embedded/picture.schema';
import { IPicture } from './models/interfaces/picture.interface';
import { IAuthor } from './models/interfaces/author.interface';
import { Author, AuthorDocument, AuthorSchema } from './models/schemas/embedded/author.schema';
import { CropAreaDTO } from './models/dtos/commons/crop-area.dto';
import { StorageService } from './services/storage.service';
import { IDParams } from './models/dtos/commons/id.param';
import { UserType } from './models/enums/user-type.enum';
import { IAMService } from './services/iam.service';

export {
    /**
     * Services
     */
    SessionsService,
    DictionariesService,
    SMTPService,
    StorageService,
    IAMService,

    /**
     * Rest Wrapper
     */
    RESTError,
    RESTPage,
    RESTPaging,
    RESTQueryParams,
    RESTRequestBody,
    RESTResponseBody,
    RESTResult,
    RESTSort,
    SortDirection,
    IMongoPage,

    /**
     * Guards
     */
    AccessTokenGuard,
    PermissionGuard,
    AllowPermissions,

    /**
     * Consts
     */
    APP__PORT,
    APP__DEBUG,
    APP__ACCESS_TOKEN,
    APP__MONGO__CS,
    DL_SESSIONS__MONGO__CS,
    MONGO__CS,
    REDIS__CS,
    KAFKA__BROKERS,

    /**
     * Enums
     */
    RecordState,
    ContactType,
    UserVerificationLevel,
    UserType,
    WeekDay,
    GroupState,
    S3FileState,
    FileLogInitType,
    AttachementType,

    /**
     * DTOs
     */
    I18nDTO,
    CropAreaDTO,
    IDParams,

    /**
     * Interfaces
     */
    IIdentifable,
    IBase,
    IRecord,
    II18n,
    IDictionary,
    IPermission,
    IRole,
    IContact,
    IPerson,
    IResidence,
    ISessionUser,
    ISession,
    IUser,
    IParent,
    IGroupEmbedded,
    IStudentEmbedded,
    ITeacher,
    IGroupSession,
    ITime,
    ISMTPAPIConfig,
    IS3,
    ILogInitiator,
    IS3File,
    IFileState,
    IFileRecord,
    IFileMeta,
    IFileLog,
    IUserMeta,
    ICropArea,
    IAttachement,
    IPicture,
    IAuthor,
    IFileEmbedded,

    /**
     * Schemas & Documents
     */
    Identifable,
    IdentifableSchema,
    IdentifableDocument,
    Base,
    BaseSchema,
    BaseDocument,
    Record,
    RecordSchema,
    RecordDocument,
    I18n,
    I18nSchema,
    I18nDocument,
    Residence,
    ResidenceSchema,
    ResidenceDocument,
    Contact,
    ContactSchema,
    ContactDocument,
    Person,
    PersonSchema,
    PersonDocument,
    DictionaryEmbedded,
    DictionaryEmbeddedSchema,
    DictionaryEmbeddedDocument,
    SessionUser,
    SessionUserSchema,
    SessionUserDocument,
    Session,
    SessionSchema,
    SessionDocument,
    GroupSessionSchema,
    GroupSessionDocument,
    GroupSession,
    TeacherSchema,
    TeacherDocument,
    Teacher,
    StudentEmbeddedSchema,
    StudentEmbeddedDocument,
    StudentEmbedded,
    TimeSchema,
    TimeDocument,
    Time,
    S3,
    S3Schema,
    S3Document,
    LogInitiator,
    LogInitiatorSchema,
    LogInitiatorDocument,
    FileLog,
    FileLogSchema,
    FileLogDocument,
    FileMeta,
    FileMetaSchema,
    FileMetaDocument,
    FileState,
    FileStateSchema,
    FileStateDocument,
    FileRecord,
    FileRecordSchema,
    FileRecordDocument,
    S3File,
    S3FileSchema,
    S3FileDocument,
    FileEmbedded,
    FileEmbeddedSchema,
    FileEmbeddedDocument,
    CropArea,
    CropAreaSchema,
    CropAreaDocument,
    Attachement,
    AttachementSchema,
    AttachementDocument,
    Picture,
    PictureSchema,
    PictureDocument,
    Author,
    AuthorSchema,
    AuthorDocument,

    /**
     * Filters
     */
    BaseExceptionsFilter,

    /**
     * Factories
     */
    validationExceptionFactory,

    /**
     * Exceptions
     */
    HandledException,

    /**
     * Decorators
     */
    IsMapOfStrings,

    /**
     * Transformers
     */
    transformers,

    /**
     * Pipes
     */
    MongoIdPipe,
    MongoIdsPipe,

    /**
     * Modules
     */
    DLCommonsModule,
};
