import { HttpService } from '@nestjs/axios';
import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { RESTResponseBody, RESTError, RESTResult } from '../models/dtos/wrappers/rest-wrapper.class';
import { firstValueFrom } from 'rxjs';
import { UserVerificationLevel } from '../models/enums/user-verifiation-level.enum';
import { UserType } from '../models/enums/user-type.enum';
import { IUser } from '../models/interfaces/user.interface';

@Injectable()
export class IAMService {
    //#region Private Members
    private _baseUrl: string;
    //#endregion

    //#region Constructor
    constructor(private readonly _httpService: HttpService, private readonly _configService: ConfigService) {
        this._baseUrl = this._configService.getOrThrow('DL_IAM__REST__BASE_URL');
    }
    //#endregion

    //#region Public API
    public async list(firstName: string, lastName: string, types: UserType[], verificationLevels: UserVerificationLevel[]): Promise<RESTResult<IUser[]>> {
        try {
            const queryParams = new URLSearchParams();
            if (firstName) {
                queryParams.append('firstName', firstName);
            }

            if (lastName) {
                queryParams.append('lastName', lastName);
            }

            if (types && types.length) {
                types.forEach((type) => queryParams.append('types', type));
            }

            if (verificationLevels && verificationLevels.length) {
                verificationLevels.forEach((level) => queryParams.append('verificationLevels', level));
            }

            const qs = queryParams.toString();

            const endpoint = `${this._baseUrl}/v1/users?${qs}`;
            const response$ = this._httpService.get<RESTResponseBody<IUser[]>>(endpoint);
            const response = await firstValueFrom(response$);

            if (response.data.errors) {
                throw new RESTError('iam_api_error', response.data.errors);
            }

            if (!response.data || !response.data.result) {
                throw new RESTError('iam_api_error', 'empty response');
            }

            return response.data.result;
        } catch (e) {
            throw RESTError.createFromCatch(e, { keyword: 'unknown', debug: e });
        }
    }

    public async read(id: string) : Promise<RESTResult<IUser>> {
        try {
            const endpoint = `${this._baseUrl}/v1/users/${id}`;
            const response$ = this._httpService.get<RESTResponseBody<IUser>>(endpoint);
            const response = await firstValueFrom(response$);

            if (response.data.errors) {
                throw new RESTError('iam_api_error', response.data.errors);
            }

            if (!response.data || !response.data.result) {
                throw new RESTError('iam_api_error', 'empty response');
            }

            return response.data.result;
        } catch (e) {
            throw RESTError.createFromCatch(e, { keyword: 'unknown', debug: e });
        }
    }
    //#endregion
}
