import { Injectable } from '@nestjs/common';
import * as FormData from 'form-data';
import { ConfigService } from '@nestjs/config';
import axios, { AxiosRequestConfig, AxiosResponse } from 'axios';
import internal from 'stream';
import { RESTError } from '../models/dtos/wrappers/rest-wrapper.class';
import { IFileRequest } from '../models/interfaces/dl-storage.interface';
import { IS3File } from '../models/interfaces/s3.interface';

@Injectable()
export class StorageService {
    //#region Private Members
    private readonly _storageApiUrl: string;
    private readonly _accessToken: string;
    private readonly _bucketName: string;
    //#endregion

    //#region Constructor
    constructor(private readonly _configsService: ConfigService) {
        this._storageApiUrl = this._configsService.getOrThrow('DL_STORAGE__REST__BASE_URL');
        this._accessToken = this._configsService.getOrThrow('DL_STORAGE__REST__ACCESS_TOKEN');
        this._bucketName = this._configsService.getOrThrow('DL_STORAGE__REST__BUCKET_NAME');
    }
    //#endregion

    //#region Public API
    public async upload(body: IFileRequest, autoCommit: boolean) {
        const form = new FormData();
        form.append('file', body.file, {
            filename: body.fileName,
            contentType: body.mimeType,
            knownLength: body.size,
        });

        let url = `${this._storageApiUrl}/bucket/${this._bucketName}/upload`;
        if (autoCommit) {
            url += '?autoCommit=true';
        }

        const response = await axios.postForm(url, form, this._axiosStandardConfig);
        return this._handleAxiosResponse(response, 'upload_failed');
    }

    public async download(fileId: string): Promise<AxiosResponse<internal.PassThrough>> {
        try {
            return axios.get<internal.PassThrough>(`${this._storageApiUrl}/bucket/${this._bucketName}/file/${fileId}/download`, this._axiosStreamConfig);
        } catch (error) {
            throw new RESTError('storage_error', error);
        }
    }

    public async getMeta(fileId: string, bucketName: string): Promise<IS3File> {
        const response = await axios.get(`${this._storageApiUrl}/bucket/${bucketName}/file/${fileId}/meta`, this._axiosStandardConfig);
        return this._handleAxiosResponse(response, 'get_meta_failed');
    }
    //#endregion

    //#region Private API
    private _handleAxiosResponse(response: AxiosResponse, errorKeyword: string): IS3File {
        if (response?.data && response.data?.result && response.data.result?.data) {
            return response.data.result.data;
        } else {
            throw { error: { keyword: errorKeyword, debug: response.data } };
        }
    }

    private get _axiosStandardConfig(): AxiosRequestConfig {
        return {
            headers: {
                'X-Access-Token': this._accessToken,
            },
        };
    }

    private get _axiosStreamConfig(): AxiosRequestConfig {
        return {
            responseType: 'stream',
            headers: {
                'X-Access-Token': this._accessToken,
            },
        };
    }
    //#endregion
}
